local BS = AceLibrary("Babble-Spell-2.2")
local SEA = AceLibrary("SpecialEvents-Aura-2.0")

Damnation = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceEvent-2.0")

local MODE_ON = "on"
local MODE_AUTO = "auto"
local MODE_OFF = "off"

local mode
local class

local function SetMode(newMode, verbose)
	if newMode == MODE_ON or newMode == MODE_AUTO or newMode == MODE_OFF then
		DamnationDBPC = newMode
		mode = newMode
	else
		error(format("Illegal mode! New mode: %s", newMode))
		DamnationDBPC = MODE_AUTO
		mode = MODE_AUTO
		verbose = true
	end
	local self = Damnation
	self:UnregisterAllEvents()
	if mode == MODE_ON then
		self:RegisterEvent("SpecialEvents_PlayerBuffGained", "BuffGained")
		if verbose then self:Print("Mode is set to \"|cff00ff00on|r\". Will remove BoS as soon as it's gained.") end
	elseif mode == MODE_AUTO then
		if class == "DRUID" then
			self:RegisterEvent("SpecialEvents_PlayerBuffGained", "BuffGained")
			self:RegisterEvent("UPDATE_SHAPESHIFT_FORM", "Damn")
			if verbose then self:Print("Mode is set to \"|cff00ff00auto|r\". Will remove BoS when you're in Bear Form.") end
		elseif class == "PALADIN" then
			self:RegisterEvent("SpecialEvents_PlayerBuffGained", "BuffGained")
			if verbose then self:Print("Mode is set to \"|cff00ff00auto|r\". Will remove BoS when you have Righteous Fury active.") end
		elseif class == "WARRIOR" then
			self:RegisterEvent("SpecialEvents_PlayerBuffGained", "BuffGained")
			self:RegisterEvent("UPDATE_SHAPESHIFT_FORM", "Damn")
			if verbose then self:Print("Mode is set to \"|cff00ff00auto|r\". Will remove BoS when you're in Defensive Stance.") end
		else
			return SetMode(MODE_OFF, verbose)
		end
	elseif mode == MODE_OFF then
		if verbose then self:Print("Mode is set to \"|cffff0000off|r\". Won't touch your BoS.") end
		return
	end
	self:Damn()
end

function Damnation:OnInitialize()
	class = select(2, UnitClass("player"))
	self:RegisterChatCommand({'/damn', '/damnation'}, function(msg)
		if msg == "" then
			if class == "DRUID" or class == "PALADIN" or class == "WARRIOR" then
				self:Print("Use /damn [on|auto|off] to change modes.")
			else
				self:Print("Use /damn [on|off] to change modes.")
			end
			return
		end
		msg = msg:lower()
		if msg == "on" or msg == "auto" or msg == "off" then
			return SetMode(msg, true)
		end
		self:Print(format("Unknown command: \"%s\"", msg))
	end)
end

function Damnation:OnEnable()
	if DamnationDBPC and type(DamnationDBPC) == "string" and (DamnationDBPC == MODE_ON or DamnationDBPC == MODE_AUTO or DamnationDBPC == MODE_OFF) then
		SetMode(DamnationDBPC)
	else
		self:Print("First time startup of Damnation!")
		SetMode(MODE_AUTO, true)
		if class == "DRUID" or class == "PALADIN" or class == "WARRIOR" then
			self:Print("Use /damn [on|auto|off] to change modes.")
		else
			self:Print("Use /damn [on|off] to change modes.")
		end
	end
end

function Damnation:BuffGained(name, index, count, icon, rank, duration, timeLeft)
	if name == BS["Greater Blessing of Salvation"] or name == BS["Blessing of Salvation"] then
		if mode == MODE_ON then
			CancelPlayerBuff(GetPlayerBuff(index, "HELPFUL"))
		elseif mode == MODE_AUTO then
			if class == "DRUID" then
				if GetShapeshiftForm(true) == 1 then
					CancelPlayerBuff(GetPlayerBuff(index, "HELPFUL"))
				end
			elseif class == "WARRIOR" then
				if GetShapeshiftForm(true) == 2 then
					CancelPlayerBuff(GetPlayerBuff(index, "HELPFUL"))
				end
			elseif class == "PALADIN" then
				if SEA:UnitHasBuff("player", BS["Righteous Fury"]) then
					CancelPlayerBuff(GetPlayerBuff(index, "HELPFUL"))
				end
			end
		end
	elseif class == "PALADIN" and name == BS["Righteous Fury"] then
		index = SEA:UnitHasBuff("player", BS["Greater Blessing of Salvation"]) or SEA:UnitHasBuff("player", BS["Blessing of Salvation"])
		if index then
			CancelPlayerBuff(GetPlayerBuff(index, "HELPFUL"))
		end
	end
end

function Damnation:Damn()
	local index = SEA:UnitHasBuff("player", BS["Greater Blessing of Salvation"]) or SEA:UnitHasBuff("player", BS["Blessing of Salvation"])
	if not index then
		return
	end
	if mode == MODE_ON then
		CancelPlayerBuff(GetPlayerBuff(index, "HELPFUL"))
	elseif mode == MODE_AUTO then
		if class == "DRUID" then
			if GetShapeshiftForm(true) == 1 then
				CancelPlayerBuff(GetPlayerBuff(index, "HELPFUL"))
			end
		elseif class == "WARRIOR" then
			if GetShapeshiftForm(true) == 2 then
				CancelPlayerBuff(GetPlayerBuff(index, "HELPFUL"))
			end
		elseif class == "PALADIN" then
			if SEA:UnitHasBuff("player", BS["Righteous Fury"]) then
				CancelPlayerBuff(GetPlayerBuff(index, "HELPFUL"))
			end
		end
	end
end
